/*
 * grunt-git-flow
 * Copyright (c) 2016 Bruno Santos
 * Licensed under the MIT license.
 */

'use strict';

var simpleGit = require('simple-git');
var semver = require('semver');
var mustache = require('mustache');

var path = process.cwd();
var pkg = require(path + '/package.json');
var tmpl = {

  'create-feature': '' +
    'echo Creating a feature branch && ' +
    'git checkout -b feature-{{name}} develop',

  'incorporate-feature': '' +
    'echo Incorporating a finished feature on develop && ' +
    'git checkout develop && ' +
    'git merge --no-ff {{name}} && ' +
    'git branch -d {{name}} && ' +
    'git push origin develop',

  'create-release': '' +
    'echo Creating a release branch for {{nextReleaseVersion}} && ' +
    'git checkout -b release-{{nextReleaseVersion}} develop && ' +
    'sed -i "" -e "s/{{currentVersion}}/{{nextReleaseVersion}}.0/g" package.json && ' +
    'git commit -a -m "bumped version number to {{nextReleaseVersion}}.0" &&' +
    'git push -u origin release-{{nextReleaseVersion}}',

  'finish-release': '' +
    'echo Finishing a release branch for {{name}} && ' +
    'git checkout master && ' +
    'git merge --no-ff {{name}} && ' +
    'git tag v{{currentVersion}} && ' +
    'git checkout develop && ' +
    'git merge --no-ff {{name}}',

  'create-hotfix': '' +
    'echo Creating the {{patchVersion}} hotfix branch && ' +
    'git checkout -b hotfix-{{patchVersion}} {{name}} && ' +
    'sed -i "" -e "s/{{currentVersion}}/{{patchVersion}}/g" package.json && ' +
    'git commit -a -m "bumped version number to {{patchVersion}}"',

  'finish-hotfix': '' +
    'echo Finishing a hotfix branch && ' +
    'git checkout release-{{currentReleaseVersion}} && ' +
    'git merge --no-ff {{name}} && ' +
    'git tag v{{currentVersion}} && ' +
    'git checkout develop && ' +
    'git merge --no-ff {{name}} && ' +
    'git branch -d {{name}}'
};

module.exports = function(grunt) {

  grunt.registerMultiTask('git-flow', 'git flow', function() {
    var options = this.options();
    //console.log('opts', options);

    // tell grunt this task is asynchronous
    var done = this.async();

    /**
     * start
     * @return {void}
     */
    function start(branch) {
      //console.log('start', branch);

      // define jobs for grunt shell
      grunt.config.merge({
        shell: {
          options: {
            gruntLogHeader: false,
            stderr: false
          },
          'create-feature': {},
          'incorporate-feature': {},
          'create-release': {},
          'finish-release': {},
          'create-hotfix': {},
          'finish-hotfix': {}
        }
      });

      // set up prompt
      grunt.config.merge({
        pkg: grunt.file.readJSON('package.json'),
        prompt: {
          options: {
            gruntLogHeader: false
          },
          bump: {
            options: {
              questions: [{
                config: 'action',
                type: 'list',
                message: 'Current version is ' + '<%= pkg.version %>',
                choices: [{
                  value: 'create-feature',
                  name: 'Create a feature branch'
                }, {
                  value: 'incorporate-feature',
                  name: 'Incorporate a finished feature on develop'
                }, {
                  value: 'create-release',
                  name: 'Create a release branch'
                }, {
                  value: 'finish-release',
                  name: 'Finish a release branch'
                }, {
                  value: 'create-hotfix',
                  name: 'Create the hotfix branch'
                }, {
                  value: 'finish-hotfix',
                  name: 'Finish a hotfix branch'
                }]
              }, {
                config: 'name',
                type: 'input',
                message: 'Name for the feature branch',
                when: function(answers) {
                  return answers.action === 'create-feature';
                }
              }, {
                config: 'name',
                type: 'list',
                message: 'Feature branch to incorporate',
                default: branch.feature[0],
                choices: branch.feature,
                when: function(answers) {
                  return answers.action === 'incorporate-feature';
                }
              }, {
                config: 'name',
                type: 'list',
                message: 'Select a release branch to finish',
                default: branch.release[0],
                choices: branch.release,
                when: function(answers) {
                  return answers.action === 'finish-release';
                }
              }, {
                config: 'name',
                type: 'list',
                message: 'Select a release branch to hotfix',
                default: branch.release[0],
                choices: branch.release,
                when: function(answers) {
                  return answers.action === 'create-hotfix';
                }
              }, {
                config: 'name',
                type: 'list',
                message: 'Select a hotfix branch to finish',
                default: branch.hotfix[0],
                choices: branch.hotfix,
                when: function(answers) {
                  return answers.action === 'finish-hotfix';
                }
              }],
              then: function(result) {
                //console.log('result', result);

                var currentVersion = pkg.version;
                var majorVersion = semver.inc(currentVersion, 'major');
                var minorVersion = semver.inc(currentVersion, 'minor');
                var patchVersion = semver.inc(currentVersion, 'patch');
                var currentReleaseVersion = currentVersion.substr(
                  0,
                  currentVersion.lastIndexOf('.')
                );
                var nextReleaseVersion = minorVersion.substr(
                  0,
                  minorVersion.lastIndexOf('.')
                );

                var str = mustache.render(tmpl[result.action], {
                  // value from user input
                  name: result.name,

                  // current version from package.json
                  currentVersion: currentVersion,

                  // next major version
                  majorVersion: majorVersion,

                  // next minor version
                  minorVersion: minorVersion,

                  // next patch version
                  patchVersion: patchVersion,

                  // current release version like major.minor
                  currentReleaseVersion: currentReleaseVersion,

                  // next release version like major.minor
                  nextReleaseVersion: nextReleaseVersion
                });

                //console.log(str);

                grunt.config.set('shell.' + result.action + '.command', str);
                grunt.task.run('shell:' + result.action);
              }
            }
          }
        }
      });

      // hide tasks headers
      require('grunt-log-headers')(grunt);

      grunt.loadNpmTasks('grunt-prompt');
      grunt.loadNpmTasks('grunt-shell');

      grunt.task.run('prompt');

      done();
    }

    // get branch list
    simpleGit(path + '/').branch(function(err, obj) {
      var branch = {
        feature: [],
        release: [],
        hotfix: []
      };

      for (var i = 0; i < obj.all.length; i++) {
        var b = obj.all[i];

        // skip origin
        if (b.indexOf('/origin/') !== -1) {
          continue;
        }

        if (b.indexOf('feature') !== -1) {
          branch.feature.push(b);
        } else if (b.indexOf('release') !== -1) {
          branch.release.push(b);
        } else if (b.indexOf('hotfix') !== -1) {
          branch.hotfix.push(b);
        }
      }

      start(branch);
    });

  });

};
